// ArduCAM demo (C)2013 Lee
// web: http://www.ArduCAM.com
// This program is a demo of how to use most of the functions
// of the library with a supported camera modules.
//
// This demo was made for Omnivision OV2640 sensor,
// Capture and buffer the image to FIFO then store to Micro SD/TF card with BMP format.
// This program requires the ArduCAM V2.0.0 (or above) library and Rev.B above ArduCAM shield
//

//#include <UTFT.h>
#include <SD.h>
#include <Wire.h>
#include <ArduCAM.h>
#include <avr/pgmspace.h>

const char bmp_header[54] PROGMEM =
{
      0x42, 0x4D, 0x36, 0x58, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x36, 0x00, 0x00, 0x00, 0x28, 0x00, 
      0x00, 0x00, 0x40, 0x01, 0x00, 0x00, 0xF0, 0x00, 0x00, 0x00, 0x01, 0x00, 0x10, 0x00, 0x00, 0x00, 
      0x00, 0x00, 0x00, 0x58, 0x02, 0x00, 0xC4, 0x0E, 0x00, 0x00, 0xC4, 0x0E, 0x00, 0x00, 0x00, 0x00, 
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
  
#define SD_CS 9 
#define BMPIMAGEOFFSET 54

//UTFT(byte model, int RS, int WR, int RD, int CS)
//UTFT myGLCD(ITDB32S,A2,A1,A0,10);   // Remember to change the model parameter to suit your display module!
//ArduCAM(byte model,int RS, int WR, int RD, int REG_CS, int FIFO_CS)
ArduCAM myCAM(OV2640,A2,A1,A0,A3,10);  // Remember to change the model parameter to suit your canera module!

void setup()
{
  //Initialize I2C Bus
  Wire.begin();   
  //Switch to FIFO Mode
  myCAM.write_reg(ARDUCHIP_TIM, MODE_MASK);
  //Initialize Camera Module
  myCAM.InitCAM();
  //Initialize SD Card
  if (!SD.begin(SD_CS)) 
  {
    //while (1);		//If failed, stop here
  }
}

void loop()
{
  char str[8];
  File outFile;
  char VH,VL;
  int i,j = 0;
  
  static int k = 0;
  uint8_t temp;
  uint8_t start_capture = 0;
  
  //Wait trigger from shutter buttom   
  if(myCAM.read_reg(ARDUCHIP_TRIG) & SHUTTER_MASK)	
  {
    //Wait until buttom released
    while(myCAM.read_reg(ARDUCHIP_TRIG) & SHUTTER_MASK);
    start_capture = 1;
  }
  
  //Start capture when detect a valid shutter press  
  if(start_capture)
  {
    //Flush the FIFO 
    myCAM.flush_fifo();		 
    //Start capture
    myCAM.start_capture();		 
  }
  
  //Polling the capture done flag
  if(myCAM.read_reg(ARDUCHIP_TRIG) & CAP_DONE_MASK)
  {
    //Construct a file name
    k = k + 1;
    itoa(k, str, 10); 
    strcat(str,".bmp");
    
    //Open the new file
    outFile = SD.open(str,FILE_WRITE);
    if (! outFile) 
      return;
      
    //Write the BMP header
    for( i = 0; i < 54; i++)
    {
      char ch = pgm_read_byte(&bmp_header[i]);
      outFile.write((uint8_t*)&ch,1);
    }
	
    //Enable FIFO
    myCAM.enable_fifo();
    //Read the first dummy byte from FIFO
    temp = myCAM.read_fifo();
    //Read 320x240x2 byte from FIFO
    for(i = 0; i < 240; i++)
      for(j = 0; j < 320; j++)
    {
        VH = myCAM.read_fifo();
        VL = myCAM.read_fifo();
        //RGB565 to RGB555 Conversion
        VL = (VH << 7) | ((VL & 0xC0) >> 1) | (VL & 0x1f);
        VH = VH >> 1;
        //Write image data to file
        outFile.write(VL); 
        outFile.write(VH);
    }
    //Disable FIFO when all the image data is saved to the file
    myCAM.disable_fifo();
    //Close the file  
    outFile.close(); 
    
    //Clear the capture done flag 
    myCAM.clear_fifo_flag();
    //Clear the start capture flag
    start_capture = 0;
  }
}
