// ArduCAM demo (C)2013 Lee
// web: http://www.ArduCAM.com
// This program is a demo of how to use most of the functions
// of the library with a supported camera modules.
//
// This demo was made for Omnivision OV2640 sensor.
// 1. Preview the live video on LCD Screen.
// 2. Capture and buffer the image to FIFO. 
// 3. Store the image to Micro SD/TF card with BMP format.
// This program requires the ArduCAM V2.0.0 (or above) library and Rev.B ArduCAM shield
//

#include <UTFT.h>
#include <SD.h>
#include <Wire.h>
#include <ArduCAM.h>
#include <avr/pgmspace.h>

const char bmp_header[54] PROGMEM =
{
      0x42, 0x4D, 0x36, 0x58, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x36, 0x00, 0x00, 0x00, 0x28, 0x00, 
      0x00, 0x00, 0x40, 0x01, 0x00, 0x00, 0xF0, 0x00, 0x00, 0x00, 0x01, 0x00, 0x10, 0x00, 0x00, 0x00, 
      0x00, 0x00, 0x00, 0x58, 0x02, 0x00, 0xC4, 0x0E, 0x00, 0x00, 0xC4, 0x0E, 0x00, 0x00, 0x00, 0x00, 
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
  
#define SD_CS 9 
#define BMPIMAGEOFFSET 54

//UTFT(byte model, int RS, int WR, int RD, int CS)
UTFT myGLCD(ITDB32S,A2,A1,A0,10);   // Remember to change the model parameter to suit your display module!
//ArduCAM(byte model,int RS, int WR, int RD, int REG_CS, int FIFO_CS)
ArduCAM myCAM(OV2640,A2,A1,A0,A3,10);  // Remember to change the model parameter to suit your canera module!

void setup()
{
  //Initialize I2C Bus
  Wire.begin();   
  //Switch to LCD Mode
  myCAM.write_reg(ARDUCHIP_TIM, 0);
  //Initialize Camera Module
  myCAM.InitCAM();
  //Initialize LCD Module
  myGLCD.InitLCD();
  //Initialize SD Card
  if (!SD.begin(SD_CS)) 
  {
    //while (1);		//If failed, stop here
  }
}

void GrabImage(char* str)
{
  File outFile;
  char VH,VL;
  uint8_t temp;
  int i,j = 0;
  
  outFile = SD.open(str,FILE_WRITE);
  if (! outFile) 
    return;
    
  //Switch to FIFO Mode
  myCAM.write_reg(ARDUCHIP_TIM, MODE_MASK);
  //Flush the FIFO 
  myCAM.flush_fifo();		 
  //Start capture
  myCAM.start_capture();
  
  //Polling the capture done flag
  while(!(myCAM.read_reg(ARDUCHIP_TRIG) & CAP_DONE_MASK));
  
  //Write the BMP header
  for( i = 0; i < 54; i++)
  {
    char ch = pgm_read_byte(&bmp_header[i]);
    outFile.write((uint8_t*)&ch,1);
  }
	
  //Enable FIFO
  myCAM.enable_fifo();
  //Read the first dummy byte from FIFO
  temp = myCAM.read_fifo();
  //Read 320x240x2 byte from FIFO
  for(i = 0; i < 240; i++)
    for(j = 0; j < 320; j++)
  {
      VH = myCAM.read_fifo();
      VL = myCAM.read_fifo();
      //RGB565 to RGB555 Conversion
      VL = (VH << 7) | ((VL & 0xC0) >> 1) | (VL & 0x1f);
      VH = VH >> 1;
      //Write image data to file
      outFile.write(VL); 
      outFile.write(VH);
  }
  //Disable FIFO when all the image data is saved to the file
  myCAM.disable_fifo();
  //Close the file  
  outFile.close(); 
  
  //Clear the capture done flag 
  myCAM.clear_fifo_flag();
  
  //Switch to LCD Mode
  myCAM.write_reg(ARDUCHIP_TIM, 0);
  return;
}

void loop()
{
  char str[8];
  static int k = 0;
  uint8_t temp;
  myCAM.write_reg(ARDUCHIP_MODE, 0x01);		 //Switch to CAM
  while(1)
  {
    temp = myCAM.read_reg(ARDUCHIP_TRIG);
    if(!(temp & VSYNC_MASK))				 //New Frame is coming
    {
       myCAM.write_reg(ARDUCHIP_MODE, 0x00);    //Switch to MCU
       myGLCD.resetXY();
       myCAM.write_reg(ARDUCHIP_MODE, 0x01);    //Switch to CAM
       while(!(myCAM.read_reg(ARDUCHIP_TRIG)&0x01)); //Wait for VSYNC is gone
    }
    else if(temp & SHUTTER_MASK)
    {
       k = k + 1;
       itoa(k, str, 10); 
       strcat(str,".bmp");
       myCAM.write_reg(ARDUCHIP_MODE, 0x00);    //Switch to MCU, freeze the screen 
       GrabImage(str);
    }
  }
}



